@@ -270,7 +270,7 @@ static i_mutex_t mutex;
 
 void
 i_tiff_init(void) {
-  mutex = i_mutex_create();
+  mutex = i_mutex_new();
 }
 
 static int save_tiff_tags(TIFF *tif, i_img *im);
